/* newdbxml.p
 *
 * before running this you must have reasonably up to date dbanalys output in "dbanalys/lname.dba"!
 *
 *     proutil dbname -C dbanalys -Bp 10 > dbanalys/lname.dba		# or copy an existing output file to this name
 *
 * create XML describing storage areas and table/index/lob assignments to those storage areas
 *
 *     mpro -db /data/sports120/s2k -ld lname -p util/newdbxml.p -param "8192|tenantaware.dat|sqlselect.dat|4000000000"
 *
 * "-ld lname" allows you to use multiple "logical" database names to support multiple *.dba files and set the output file name
 *
 * parameters are in an optional pipe-delimited list:
 *     p1 = db blocksize in bytes, default is the blocksize of the connected database
 *     p2 = filename of a file containing a list (one per line) of table names and yes/no values for "tenantAware", unlisted tables are assumed to be "yes"
 *     p3 = filename of a file containing a list (one per line) of table names and character values for "sqlSelect", unlisted tables are assumed to be "Public"
 *     p4 = threshold, in bytes, to be considered a "large table", default = 2000000000 (2GB)
 *
 * notes:
 *     the "Blocksize" really ought to be named "RowsPerBlock"
 *
 */

{lib/protop.i}
{lib/protoplib.i}
{lib/dba.i}

define temp-table tt_tenantAware no-undo
  field tblName     as character
  field tenantAware as logical format "true/false"
.

define temp-table tt_sqlSelect no-undo
  field tblName   as character
  field sqlSelect as character
  index tblName-idx is unique tblName
.

define stream rptOut.


function isAware returns logical ( input tblName as character ):

  find tt_tenantAware where tt_tenantAware.tblName = tblName no-error.

  /* message "isAware():" tblName available( tt_tenantAware ) ( if available( tt_tenantAware ) then tenantAware else true ). */

  return ( if available( tt_tenantAware ) then tenantAware else true ).

end.


function SqlSelect returns character ( input tblName as character ):

  find tt_sqlSelect where tt_sqlSelect.tblName = tblName no-error.

  /* message "sqlSelect():" tblName available( tt_sqlSelect ) ( if available( tt_sqlSelect ) then sqlSelect else "Public" ). */

  return ( if available( tt_sqlSelect ) then sqlSelect else "Public" ).

end.


procedure newDBXML:

  define input parameter blkSize as integer no-undo.						/* target db block size in bytes			*/

  define variable aName       as character no-undo.						/* area name						*/
  define variable minDatCsz   as integer   no-undo initial  64.					/* data area min blocks per cluster             	*/
  define variable minIdxCsz   as integer   no-undo initial  64.					/* index area min blocks per cluster             	*/
  define variable outFileName as character no-undo.						/* area name						*/

  outFileName = substitute( "&1/&2.xml", pt_rptdir, ldbname(1) ).

  output stream rptout to value( outFileName ).

  put stream rptout unformatted '<?xml version="1.0" encoding="UTF-8"?>' skip.
  put stream rptout unformatted '<?xml-stylesheet type="text/xsl" href="./tableinfo.xsl"?>' skip.
  put stream rptout unformatted '<DBStructure>' skip.
  put stream rptout unformatted '~t<areas>' skip.

  /* generic table and index areas (misc*)
   */

  for each tblist no-lock where tblist.order > 0 and tblist.order < 999
      break by tblist.order by tblist.tsz descending by tblist.trd descending:

    if last-of( tblist.order ) then
      do:
        aName = "misc" + string( tblist.order ).
        put stream rptout unformatted substitute( '~t~t<area Name="&1_dat" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', aName, minDatCsz, tblist.order, "True" ) skip.
        put stream rptout unformatted substitute( '~t~t<area Name="&1_idx" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', aName, minIdxCsz, tblist.order, "True" ) skip.
      end.

  end.

  /* dedicated table and index areas (order = 0)
   */

  for each tblist no-lock where tblist.order = 0
      by tblist.order by tblist.tsz descending by tblist.trd descending:

    if tblist.tbl begins "_" then next.
    aName = tblist.tbl.
    put stream rptout unformatted substitute( '~t~t<area Name="&1_dat" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', aName, 512, idealRPB( blkSize, tblist.tsz / tblist.recs ), "True" ) skip.
    put stream rptout unformatted substitute( '~t~t<area Name="&1_idx" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', aName, 512, 1, "True" ) skip.

  end.

  /* LOB areas (always dedicated)
   */

  for each lxlist no-lock where lxlist.lid > 0:

    find tblist where tblist.tid = lxlist.lid.
    put stream rptout unformatted substitute( '~t~t<area Name="&1_lob" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', replace( lxlist.lob, ".", "_" ), 512, 64, "True" ) skip.

  end.

  /* default areas
   */

  put stream rptout unformatted substitute( '~t~t<area Name="DefaultDataArea" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', "", 8, 64, "False" ) skip.
  put stream rptout unformatted substitute( '~t~t<area Name="DefaultIndexArea" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', "", 8, 1, "False" ) skip.
  put stream rptout unformatted substitute( '~t~t<area Name="DefaultLobArea" ClusterSize="&2" BlockSize="&3" TenantAware="&4"/>', "", 8, 64, "False" ) skip.

  put stream rptout unformatted '~t</areas>' skip.

  /* table, index, and LOB to area assignments
   */

  put stream rptout unformatted '~t<tables>' skip.

  for each tblist no-lock where tblist.tbl <> ""
      by tblist.order by tblist.tsz descending by tblist.trd descending:

    if tblist.tbl begins "_" then next.

    if tblist.order = 0 then
      aName = tblist.tbl.
     else if tblist.order < 999 then
      aName = "misc" + string( tblist.order ).
     else
      aName = "unused".

    put stream rptout unformatted substitute( '~t~t<table Database="&1" Name="&2" TenantAware="&3" Area="&4_dat" IndexArea="&5_idx" SqlSelect="&6"',
      entry( num-entries( pdbname(1), "/" ), pdbname(1), "/" ),
      tblist.tbl,
      trim( string( isAware( tblist.tbl ), "True/False" )),
      aName,
      aName,
      sqlSelect( tblist.tbl )
    ).

    find first lxlist no-lock where entry( 1, lxlist.lob, "." ) = tbl no-error.
    if not available lxlist then
      put stream rptout unformatted '/>' skip.
     else
      do:
        put stream rptout unformatted '>' skip.
        for each lxlist no-lock where entry( 1, lxlist.lob, "." ) = tbl:
          find _field no-lock where _field._file-recid = recid( _file ) and _field._field-name = entry( 2, lxlist.lob, "." ) no-error.
          if not available( _field ) then
            do:
              message now "could not find _field for" lxlist.lob + ", skipping".
              next.
            end.
          put stream rptout unformatted substitute( '~t~t~t<field Name="&1" Type="&2" Area="&3"/>', entry( 2, lxlist.lob, "." ), _field._data-type, replace( entry( 2, lxlist.lob, "." ), ".", "_" )) skip.
        end.
        put stream rptout unformatted '~t~t</table>' skip.
      end. 

  end.

  put stream rptout unformatted '~t</tables>' skip.
  put stream rptout unformatted '</DBStructure>' skip.

  output stream rptout close.

  message now "XML output is in:" outFileName.

  return.

end.


/* main
 *
 */

define variable numParams as integer   no-undo.
define variable blkSize   as integer   no-undo.
define variable pBlkSize  as integer   no-undo.
define variable lrgTable  as int64     no-undo.
define variable fileName  as character no-undo.
define variable ok        as logical   no-undo.
define variable i         as integer   no-undo.
define variable j         as integer   no-undo.

numParams = num-entries( session:parameter, "|" ).

run lib/protoplib.p persistent.						/* library of useful nondb related functions					*/
run lib/vstlib.p    persistent.						/* library of useful db related functions					*/
run lib/getdba.p    persistent.						/* library to read and parse dbanalys						*/
run lib/protop-cfg.p.							/* initialize protop global variables (specifically pt_rptdir)			*/

run dba_init( output blkSize, output fileName, output ok ).		/* make sure that dbanalys exists, has been imported and analyzed		*/

genXML: do:

  if numParams >= 1 and entry( 1, session:parameter, "|" ) <> "" then	/* block size									*/
    do:
      pBlkSize = integer( entry( 1, session:parameter, "|" )) no-error.
      if pBlkSize = 4096 or pBlkSize = 8192 then
        blkSize = pBlkSize.
       else
        do:
          message now "Block size of" pBlkSize "is not valid.  Block size must be 4096 or 8192." view-as alert-box information.
          leave genXML.
        end.
    end.

  if numParams >= 2 and entry( 2, session:parameter, "|" ) <> "" then	/* "tenantAware" list								*/
    do:
      if search( entry( 2, session:parameter, "|" )) = ? then
        do:
          message now 'Cannot find "TenantAware" input file:' entry( 2, session:parameter, "|" ) view-as alert-box information.
          leave genXML.
        end.
       else
        do:
          input from value( entry( 2, session:parameter, "|" )).
          ta: repeat:
            create tt_tenantAware.
            do on error undo, leave ta on endkey undo, leave ta:
              import tt_tenantAware.
            end.
            i = i + 1.
          end.
          input close.
        end.
    end.

  if numParams >= 3 and entry( 3, session:parameter, "|" ) <> "" then	/* sqlSelect list								*/
    do:
      if search( entry( 3, session:parameter, "|" )) = ? then
        do:
          message now 'Cannot find "SqlSelect" input file:' entry( 3, session:parameter, "|" ) view-as alert-box information.
          leave genXML.
        end.
       else
        do:
          input from value( entry( 3, session:parameter, "|" )).
          ss: repeat:
            create tt_sqlSelect.
            do on error undo, leave ss on endkey undo, leave ss:
              import tt_sqlSelect.
            end.
            j = j + 1.
          end.
          input close.
        end.
    end.

  if numParams >= 4  and entry( 4, session:parameter, "|" ) <> "" then	/* lrgTable threshold								*/
    lrgTable = int64( entry( 4, session:parameter, "|" )) no-error.

  if lrgTable = ? or lrgTable <= 0 then
    lrgTable = 2000000000.


  /* classify the table
   */

  for each tblist:
            
    if       tblist.recs = 0         then tblist.order = 999.			/* unused					*/
     else if tblist.tsz >= lrgTable  then tblist.order = 0.			/* large table, dedicated area			*/
     else tblist.order   = idealRPB( blkSize, ( tblist.tsz / tblist.recs )).	/* ordinary table, misc RPB sized area		*/

  end.

/*
 *   for each tblist where tid >= 0:
 *     display
 *       tblist.order
 *       tblist.tid
 *       tblist.tbl
 *       tblist.tsz
 *       tblist.recs
 *     .
 *   end.
 *   pause.
 */

  message now "BlockSize =" blkSize i "tenantAware records and" j "sqlSelect records. lrgTable =" lrgTable.

  run newDBXML( blkSize ).							/* creates the XML				*/

end.

quit.
